library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity voterPRM is 
    generic( 
        DATA_WIDTH: integer := GENERIC_DATA_WIDTH_PARAM
    );
    port(
        DATA1 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA2 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA3 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA  : out std_logic_vector(DATA_WIDTH-1 downto 0);
        PRM_ERROR : out std_logic_vector(3 downto 0)  
    ); 
end voterPRM;


architecture voterPRM_structural of voterPRM is

    signal data_sig : std_logic_vector(DATA_WIDTH-1 downto 0);
    signal voter1_err1_sig, voter1_err2_sig, voter1_err3_sig : std_logic;  
    signal voter2_err1_sig, voter2_err2_sig, voter2_err3_sig : std_logic;  
    signal err1_cmp_sig, err2_cmp_sig, err3_cmp_sig, err_sig, err_neg_sig : std_logic;  
    
    component voter is
    generic ( 
        DATA_WIDTH: integer
    );
    port(
        DATA1 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA2 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA3 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA  : out std_logic_vector(DATA_WIDTH-1 downto 0);
        ERR1  : out std_logic; 
        ERR2  : out std_logic; 
        ERR3  : out std_logic 
    ); 
    end component;

    component mux is
    generic ( 
        DATA_WIDTH: integer
    );
    port(
        DATA1 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA2 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA3 : in std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA_OUT1 : out std_logic_vector(DATA_WIDTH-1 downto 0);
        DATA_OUT2 : out std_logic_vector(DATA_WIDTH-1 downto 0);
        ERR1  : in std_logic; 
        ERR2  : in std_logic; 
        ERR3  : in std_logic  
    );
    end component;
 
begin

    voter_inst1 : voter
    generic map ( 
        DATA_WIDTH  => DATA_WIDTH)
    port map(
        DATA1 => DATA1,
        DATA2 => DATA2,
        DATA3 => DATA3,
        DATA => data_sig,
        ERR1 => voter1_err1_sig,
        ERR2 => voter1_err2_sig,
        ERR3 => voter1_err3_sig
    );
    
    voter_inst2 : voter
    generic map ( 
        DATA_WIDTH  => DATA_WIDTH)
    port map(
        DATA1 => DATA1,
        DATA2 => DATA2,
        DATA3 => DATA3,
        DATA => open,
        ERR1 => voter2_err1_sig,
        ERR2 => voter2_err2_sig,
        ERR3 => voter2_err3_sig
    );
    
    err1_cmp_sig <= voter1_err1_sig xor voter2_err1_sig; 
    err2_cmp_sig <= voter1_err2_sig xor voter2_err2_sig; 
    err3_cmp_sig <= voter1_err3_sig xor voter2_err3_sig; 
    err_sig <= err1_cmp_sig or err2_cmp_sig or err3_cmp_sig;
    err_neg_sig <= not err_sig;
    
    PRM_ERROR(0) <= err_sig;
    PRM_ERROR(1) <= voter1_err1_sig and err_neg_sig;
    PRM_ERROR(2) <= voter1_err2_sig and err_neg_sig;
    PRM_ERROR(3) <= voter1_err3_sig and err_neg_sig;

    DATA <= data_sig;
    
end voterPRM_structural;
