printHelp()
{
    echo "Spusteni skriptu"
    echo "./synthetise.sh [-h] [-s SPEED]"
    echo "-h              zobrazeni napovedy"
    echo "-s MS           rychlost citani v ms"
}

appendToWarningLog()
{
  warningCount=`cat xst.log | grep -c WARNING`
  if [ $warningCount -ne 0 ]
  then
    echo "Komponenta "$1 >> ../../warning.log
    echo "========================" >> ../../warning.log
    cat xst.log | grep WARNING >> ../../warning.log
    echo "" >> ../../warning.log
  fi
}


genericDevice="xc5vsx50t"
genericDeviceFamily="virtex5"
genericPackage="ff1136"


echo "Jednoduchy citac se start/stop funkci"
echo "====================================="
echo "Citani od 0 do 255 se zobrazuje na ledkach. Reset se provadi tlacitkem SW_C, start/stop tlacitkem SW_N."
echo ""

if [ $# -eq 1 ] && [ $@ == "-h" ]; then
    printHelp
    exit 1
fi 

genericSpeed=500

while getopts ":s:" parameter
do
    case "$parameter" in
        s) genericSpeed=$OPTARG;;
        \?) echo "Zadan chybny parametr!"; echo ""; printHelp; exit 1;;
    esac 
done


echo "Nastavene genericke hodnoty"
echo "Device            = $genericDevice"
echo "DeviceFamily      = $genericDeviceFamily"
echo "Package           = $genericPackage"
echo "Rychlost citani   = $genericSpeed"
echo ""


if ! [ -d ngc ]; then
    mkdir ngc
fi

if [ -d synth ]; then
    rm -r synth
fi
mkdir synth
mkdir synth/top
mkdir synth/counter

cp src/top_level.vhd synth/top
cp src/counter.vhd synth/top
cp src/math_pack.vhd synth/top
cp src/switch.vhd synth/top
cp src/chipscope_ila.vhd synth/top
cp src/chipscope_ila1.vhd synth/top
cp src/chipscope_icon.vhd synth/top
cp xst/top.xst synth/top
sed -i "s/SPEED.*:.*integer.*:=.*[0-9]*/SPEED : integer := $genericSpeed/g" synth/top/top_level.vhd

cp src/simple_counter.vhd synth/counter
cp xst/counter.xst synth/counter

cd synth/top
ls *.vhd | sed 's/\(.*\)/vhdl work \1/' > top.prj

echo "Synteza top_level zahajena..."
if xst -ifn top.xst > xst.log 
then
  echo "Synteza top_level dokoncena (W:"`cat xst.log | grep -c WARNING`")"
  appendToWarningLog "top_level"
else
  echo "Chyba pri synteze komponenty top_level, zkontrolujte synth/top/xst.log"
  exit 1
fi

cd ../..

cd synth/counter
ls *.vhd | sed 's/\(.*\)/vhdl work \1/' > counter.prj

echo "Synteza counter zahajena..."
if xst -ifn counter.xst > xst.log 
then
  echo "Synteza counter dokoncena (W:"`cat xst.log | grep -c WARNING`")"
  appendToWarningLog "counter"
else
  echo "Chyba pri synteze komponenty counter, zkontrolujte synth/counter/xst.log"
  exit 1
fi

cd ../..

cp synth/counter/simple_counter.ngc ngc
cp src/chipscope_ila.ngc ngc
cp src/chipscope_ila1.ngc ngc
cp src/chipscope_icon.ngc ngc
cp synth/top/top.ngc ngc

echo "Soubor top_level.ngc a simple_counter.ngc byly vygenerovany v adresari ngc."

